/**
  *  \file t_sstring.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "cpluslib/sstring.h"

#include "tests.h"

/* test ctors, assignment operator, assign() */
void
TestStringT::testInit()
{
//     simple_string();
//     simple_string(const charT* src);
//     simple_string(const proxy&);
//     simple_string(const simple_string& rhs);
//     simple_string(const simple_string& str, size_type pos, size_type n = npos);
//     simple_string(const charT* src, size_type n);
//     simple_string(size_type n, charT c);

//     template<class InputIterator>
//     simple_string(InputIterator begin, InputIterator end);
//     simple_string& operator=(const simple_string& rhs);
//     simple_string& operator=(const charT* rhs);
//     simple_string& operator=(charT rhs);
//     simple_string& operator=(const proxy& rhs);
//     simple_string& assign(const simple_string& str);
//     simple_string& assign(const simple_string& str, size_type pos,
//                           size_type n);
//     simple_string& assign(const charT* s, size_type n);
//     simple_string& assign(const charT* s);
//     simple_string& assign(size_type n, charT c);
//     template<class InputIterator>
//     simple_string& assign(InputIterator first, InputIterator last);
    
    static char sequence[] = "blub"; // COMPILER BUG: should not be `const char'

    simple_string s1;                          // ""
    simple_string s2 = "abc";                  // "abc"
    simple_string s3(s2 + "def");              // "abcdef"
    simple_string s4(s3, 2, 2);                // "cd"
    simple_string s5(s3, 6, 99);               // ""
    simple_string s6(s3, 2);                   // "cdef"
    simple_string s7(s3, 2, 4);                // "cdef"
    simple_string s8("foobar", 4);             // "foob"
    simple_string s9(10, 'x');                 // "xxxxxxxxxx"
    simple_string s10(0, 'a');                 // ""
    simple_string s11(sequence, &sequence[4]); // "blub"

    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), ""), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s2.c_str(), "abc"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s3.c_str(), "abcdef"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s4.c_str(), "cd"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s5.c_str(), ""), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s6.c_str(), "cdef"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s7.c_str(), "cdef"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s8.c_str(), "foob"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s9.c_str(), "xxxxxxxxxx"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s10.c_str(), ""), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s11.c_str(), "blub"), 0);

    s1 = s6;
    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), "cdef"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s6.c_str(), "cdef"), 0);
    s10 = "fasel";
    s5 = 'x';
    s11 = s2 + s3 + s4;
    TS_ASSERT_EQUALS(std::strcmp(s10.c_str(), "fasel"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s5.c_str(),  "x"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s11.c_str(), "abcabcdefcd"), 0);    
    TS_ASSERT_EQUALS(std::strcmp(s3.c_str(),  "abcdef"), 0);

    s1.assign(s2);                   // abc
    s2.assign(s2, 2, simple_string::npos); // c
    s3.assign(s11, 3, 3);            // abc
    s4.assign("quuxuum", 4);         // quux
    s5.assign("quuxuum");            // quuxuum
    s6.assign(7, '-');               // -------
    s7.assign(sequence, sequence+2); // bl

    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), "abc"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s2.c_str(), "c"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s3.c_str(), "abc"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s4.c_str(), "quux"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s5.c_str(), "quuxuum"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s6.c_str(), "-------"), 0);
    TS_ASSERT_EQUALS(std::strcmp(s7.c_str(), "bl"), 0);
}

/* test misc. accessor / manipulator functions */
void
TestStringT::testMisc()
{
//     size_type length() const { return len; }
//     const charT* data() const { return txt; }
//     size_type size() const { return len; }
//     size_type capacity() const { return len; }
//     size_type max_size() const { return npos >> 1; /* be conservative */ }
//     const charT* c_str() const;
//     void resize(size_type n, charT c);
//     void resize(size_type n) { resize(n, charT()); }
//     void swap(simple_string& rhs);
//     size_type copy(charT* s, size_type n, size_type pos = 0) const;
//     reference operator[](size_type pos) { return txt[pos]; }
//     const_reference operator[](size_type pos) const { return pos == len ? null : txt[pos]; }
//     reference at(size_type pos);
//     const_reference at(size_type pos) const;
//     bool empty() const { return len==0; }
//     void clear();
//     int compare(const simple_string& str) const;
//     int compare(size_type pos1, size_type n1,
//                 const simple_string& str) const;
//     int compare(size_type pos1, size_type n1,
//                 const simple_string& str,
//                 size_type pos2, size_type n2) const;
//     int compare(const charT* s) const;
//     int compare(size_type pos1, size_type n1,
//                 const charT* s, size_type n2 = npos) const;
//     proxy substr(size_type pos = 0, size_type n = npos) const;
    simple_string s1 = "foobar";
    simple_string s2;
    TS_ASSERT_EQUALS(s1.length(), 6);
    TS_ASSERT_EQUALS(s2.length(), 0);
    TS_ASSERT(s2.empty());
    TS_ASSERT_EQUALS(std::memcmp(s1.data(), "foobar", 6), 0);
    TS_ASSERT_EQUALS(s1.size(), 6);
    TS_ASSERT(s1.capacity() >= 6);
    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), "foobar"), 0);
    TS_ASSERT_EQUALS(s1, "foobar");
    s1.resize(3);
    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), "foo"), 0);
    TS_ASSERT_EQUALS(s1, "foo");
    s1.resize(5, 'x');
    TS_ASSERT_EQUALS(std::strcmp(s1.c_str(), "fooxx"), 0);
    TS_ASSERT_EQUALS(s1, "fooxx");
    s1.swap(s2);
    TS_ASSERT_EQUALS(s2, "fooxx");
    TS_ASSERT(s1.empty());
    {
        char buf[2];
        TS_ASSERT_EQUALS(s2.copy(buf, 2, 3), 2);
        TS_ASSERT_EQUALS(buf[0], 'x');
        TS_ASSERT_EQUALS(buf[1], 'x');
        TS_ASSERT_EQUALS(s1.copy(buf, 2, 0), 0);
    }
    swap(s1, s2);
    TS_ASSERT_EQUALS(s1, "fooxx");
    TS_ASSERT(s2.empty());
    TS_ASSERT_EQUALS(const_cast<const simple_string&>(s2)[0], 0);
    TS_ASSERT_EQUALS(s1[0], 'f');
    TS_ASSERT_EQUALS(s1[1], 'o');
    TS_ASSERT_EQUALS(s1[3], 'x');
    TS_ASSERT_EQUALS(s1.at(0), 'f');
    TS_ASSERT_EQUALS(s1.at(1), 'o');
    TS_ASSERT_EQUALS(s1.at(3), 'x');
    s1.at(4) = 'y';
    TS_ASSERT_EQUALS(s1, "fooxy");
    s1[1] = 'l';
    TS_ASSERT_EQUALS(s1, "floxy");

    s1.clear();
    TS_ASSERT(s1.empty());
    TS_ASSERT_EQUALS(s1, s2);
    TS_ASSERT_EQUALS(s1, "");
    TS_ASSERT_EQUALS("", s1);

    /* Bug: g++'s std::string's compare is broken beyond repair */
    TS_ASSERT(s1.compare("foo") < 0);
    TS_ASSERT_EQUALS(s1.compare(""), 0);
    TS_ASSERT_EQUALS(s1.compare(s2), 0);
    TS_ASSERT(s1 < "foo");
    TS_ASSERT_EQUALS(s1, "");

    s1 = "pentagondodekaeder";
    TS_ASSERT_EQUALS(s1.compare(3, 3, "tag", 3), 0);
    // if I read the CD2 correctly, the following [should] be negative:
    // st.compare(3, 3, "tag") == string(*this, 3, 3).compare(string("tag", npos))
    // (i.e. it's actually undefined behaviour -- the default argument is
    // pretty misguiding)
    TS_ASSERT(s1.compare(3, 3, "tag") < 0);
    TS_ASSERT(s1.compare("xyzzy") < 0);
    TS_ASSERT(s1.compare("dostoprimetschatjelnostj") > 0);
    TS_ASSERT_EQUALS(s1.compare(1, 4, "enta", 4), 0);
    TS_ASSERT(s1.compare(1, 4, "ente", 4) < 0);
    s2 = "makkaroni";
    TS_ASSERT_EQUALS(s1.compare(6, 2, s2, 6, 2), 0);
    --s2[6];
    TS_ASSERT(s1.compare(6, 2, s2, 6, 2) > 0);
    TS_ASSERT(s1 < "xyzzy");
    TS_ASSERT("xyzzy" > s1);
    TS_ASSERT(s1 > "mslifresser");
    TS_ASSERT(s1 != "mslifresser");
    TS_ASSERT(s1.compare("pdagoge") < 0);
    TS_ASSERT(s1.compare("pantoffel") > 0);

    s1 = "brennholzverleihsteuer";
    TS_ASSERT_EQUALS(s1.substr(5, 4), "holz");
    s2 = s1.substr(5, 4) + s1.substr(0, 5) + s1.substr(16);
    TS_ASSERT_EQUALS(s2, "holzbrennsteuer");
    s2 = simple_string(s1, 5, 4);
    TS_ASSERT_EQUALS(s2, "holz");

    TS_ASSERT_EQUALS((s2 + "foo").length(), 7);
    TS_ASSERT_EQUALS(std::strcmp((s2 + "foo").c_str(), "holzfoo"), 0);
    TS_ASSERT(!(s2 + "foo").empty());
    TS_ASSERT((simple_string() + "").empty());
    TS_ASSERT_EQUALS((s2 + "foo")[5], 'o');
    TS_ASSERT_EQUALS((s2 + "foo").compare(3, 2, "zf", 2), 0);
}

/* test append functions */
void
TestStringT::testAppend()
{
//     simple_string& operator+=(const simple_string& rhs);
//     simple_string& operator+=(const charT* rhs);
//     simple_string& operator+=(charT rhs);
//     simple_string& append(const simple_string& str);
//     simple_string& append(const simple_string& str, size_type pos, size_type n);
//     simple_string& append(const charT* s, size_type n);
//     simple_string& append(const charT* s);
//     simple_string& append(size_type n, charT c);
//     template<class InputIterator>
//     simple_string& append(InputIterator first, InputIterator last);
    simple_string s1 = "foo", s2 = "bar", s3;
    TS_ASSERT_EQUALS(s1, "foo");
    TS_ASSERT_EQUALS(s2, "bar");
    TS_ASSERT(s3.empty());

    s1 += s2;
    TS_ASSERT_EQUALS(s1, "foobar");
    TS_ASSERT_EQUALS(s2, "bar");
    s1 += "quux";
    TS_ASSERT_EQUALS(s1, "foobarquux");
    s1 += '!';
    TS_ASSERT_EQUALS(s1, "foobarquux!");
    s1 = "foo";
    TS_ASSERT_EQUALS(s1, "foo");
    s1.append(s2);
    TS_ASSERT_EQUALS(s1, "foobar");
    s1.append(s1, 3, simple_string::npos);
    TS_ASSERT_EQUALS(s1, "foobarbar");
    s1.append(s1, 0, 3);
    TS_ASSERT_EQUALS(s1, "foobarbarfoo");
    s2.append("hualp", 2);
    TS_ASSERT_EQUALS(s2, "barhu");
    s2.append("ps");
    TS_ASSERT_EQUALS(s2, "barhups");
    s2.append(3, '!');
    TS_ASSERT_EQUALS(s2, "barhups!!!");
    s3.append(4, 'x');
    TS_ASSERT_EQUALS(s3, "xxxx");
}

/* test erase / replace */
void
TestStringT::testEraseReplace()
{
//     simple_string& erase(size_type pos = 0, size_type n = npos);
//     iterator erase(iterator position);
//     iterator erase(iterator first, iterator last);

//     simple_string& replace(size_type pos1, size_type n1,
//                            const simple_string& str);
//     simple_string& replace(size_type pos1, size_type n1,
//                            const simple_string& str,
//                            size_type pos2, size_type n2);
//     simple_string& replace(size_type pos, size_type n1, const charT* s, size_type n2);
//     simple_string& replace(size_type pos, size_type n1, const charT* s);
//     simple_string& replace(size_type pos, size_type n1, size_type n2, charT c);
//     simple_string& replace(iterator i1, iterator i2, const simple_string& str);
//     simple_string& replace(iterator i1, iterator i2, const charT* s, size_type n);
//     simple_string& replace(iterator i1, iterator i2, const charT* s);
//     simple_string& replace(iterator i1, iterator i2, size_type n, charT c);
//     template<class InputIterator>
//     simple_string& replace(iterator i1, iterator i2,
//                            InputIterator j1, InputIterator j2) {
//         return replace(i1, i2, simple_string(j1, j2));
//     }
//     simple_string& insert(size_type pos1, const simple_string& str);
//     simple_string& insert(size_type pos1, const simple_string& str,
//                           size_type pos2, size_type n);
//     simple_string& insert(size_type pos, const charT* s, size_type n);
//     simple_string& insert(size_type pos, const charT* s);
//     simple_string& insert(size_type pos, size_type n, charT c);
//     iterator insert(iterator p, charT c = charT());
//     void insert(iterator p, size_type n, charT c);
//     template<class InputIterator>
//     void insert(iterator p, InputIterator first, InputIterator last) {
//         insert(p - begin(), simple_string(first, last));
//     }
    simple_string s1, s2, s3;
    s1 = "foobar";
    TS_ASSERT_EQUALS(s1, "foobar");
    s1.erase(s1.begin());
    TS_ASSERT_EQUALS(s1, "oobar");
    s1.erase();
    TS_ASSERT(s1.empty());
    TS_ASSERT_EQUALS(s1, "");
    // gcc misunderstands the following:
    // TS_ASSERT_EQUALS(s1, simple_string());
    s1 = "foobar";
    s1.erase(s1.begin()+1, s1.begin()+3);
    TS_ASSERT_EQUALS(s1, "fbar");
    s1.erase(0, 1).erase(1, 1);
    TS_ASSERT_EQUALS(s1, "br");
    s2 = "arpawocky";
    TS_ASSERT_EQUALS(s2.length(), 9);
    s2.erase(4);
    TS_ASSERT_EQUALS(s2, "arpa");
    s2.replace(1, 2, "bb");
    TS_ASSERT_EQUALS(s2, "abba");
    s2.replace(0, 2, "abracadafasel", 8);
    TS_ASSERT_EQUALS(s2, "abracadaba");
    s2.replace(8, 1, s1);
    TS_ASSERT_EQUALS(s2, "abracadabra");
    s1.replace(0, 1, s2, 4, 3);
    TS_ASSERT_EQUALS(s1, "cadr");
    s1.replace(2, 1, 10, 'x');
    TS_ASSERT_EQUALS(s1, "caxxxxxxxxxxr");
    s1.replace(s1.begin(), s1.end() - 4, "hooray");
    TS_ASSERT_EQUALS(s1, "hoorayxxxr");
    s1.replace(s1.begin(), s1.begin()+3, "x-y-ungelst", 2);
    TS_ASSERT_EQUALS(s1, "x-rayxxxr");
    s1.erase(5);
    TS_ASSERT_EQUALS(s1, "x-ray");
    s1.insert(2, "y-");
    TS_ASSERT_EQUALS(s1, "x-y-ray");
    s1.insert(1, 3, '+');
    TS_ASSERT_EQUALS(s1, "x+++-y-ray");
    s1 = "foo";
    s2 = "bar";
    TS_ASSERT_EQUALS(s1, "foo");
    TS_ASSERT_EQUALS(s2, "bar");
    s1.insert(2, s2);
    TS_ASSERT_EQUALS(s1, "fobaro");
    s2.insert(2, s1, 0, 2);
    TS_ASSERT_EQUALS(s2, "bafor");
    s2.insert(3, "weoiztsdgklvh", 2);
    TS_ASSERT_EQUALS(s2, "bafweor");
}

void
TestStringT::testFind()
{
//     size_type find (const simple_string& str, size_type pos = 0) const;
//     size_type find (const charT* s, size_type pos, size_type n) const;
//     size_type find (const charT* s, size_type pos = 0) const;
//     size_type find (charT c, size_type pos = 0) const;

//     size_type rfind(const simple_string& str, size_type pos = npos) const;
//     size_type rfind(const charT* s, size_type pos, size_type n) const;
//     size_type rfind(const charT* s, size_type pos = npos) const;
//     size_type rfind(charT c, size_type pos = npos) const;

//     size_type find_first_of(const simple_string& str, size_type pos = 0) const;
//     size_type find_first_of(const charT* s, size_type pos, size_type n) const;
//     size_type find_first_of(const charT* s, size_type pos = 0) const;
//     size_type find_first_of(charT c, size_type pos = 0) const;

//     size_type find_last_of(const simple_string& str,
//                            size_type pos = npos) const;
//     size_type find_last_of(const charT* s, size_type pos, size_type n) const;
//     size_type find_last_of(const charT* s, size_type pos = npos) const;
//     size_type find_last_of(charT c, size_type pos = npos) const;
    
//     size_type find_first_not_of(const simple_string& str, size_type pos = 0) const;
//     size_type find_first_not_of(const charT* s, size_type pos, size_type n) const;
//     size_type find_first_not_of(const charT* s, size_type pos = 0) const;
//     size_type find_first_not_of(charT c, size_type pos = 0) const;

//     size_type find_last_not_of(const simple_string& str,
//                                size_type pos = npos) const;
//     size_type find_last_not_of(const charT* s, size_type pos, size_type n) const;
//     size_type find_last_not_of(const charT* s, size_type pos = npos) const;
//     size_type find_last_not_of(charT c, size_type pos = npos) const;
    simple_string s1 = "the quick brown fox jumps over the lazy dog";
    simple_string s2 = "foobar";
    simple_string s3("dog", 4);

    TS_ASSERT_EQUALS(s3.length(), 4);
    TS_ASSERT_EQUALS(s1.find("quick"), 4);
    TS_ASSERT_EQUALS(s1.find("quickie", 0, 5), 4);
    TS_ASSERT_EQUALS(s1.find("quickie"), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find('e'), 2);
    TS_ASSERT_EQUALS(s1.find('e', 10), 28);
    TS_ASSERT_EQUALS(s1.find("the"), 0);
    TS_ASSERT_EQUALS(s1.find("the", 10), 31);
    TS_ASSERT_EQUALS(s1.find("the", 31), 31);
    TS_ASSERT_EQUALS(s1.find("the", 32), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find("the", 42), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find(s3), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find(s3.substr(0, 3)), 40);
    TS_ASSERT_EQUALS(s3.find(char(0)), 3);

    TS_ASSERT_EQUALS(s1.rfind("quick"), 4);
    TS_ASSERT_EQUALS(s1.rfind("quickie", simple_string::npos, 5), 4);
    TS_ASSERT_EQUALS(s1.rfind("quickie"), simple_string::npos);
    TS_ASSERT_EQUALS(s1.rfind('e'), 33);
    TS_ASSERT_EQUALS(s1.rfind('e', 10), 2);
    TS_ASSERT_EQUALS(s1.rfind("the"), 31);
    TS_ASSERT_EQUALS(s1.rfind("the", 10), 0);
    TS_ASSERT_EQUALS(s1.rfind("the", 31), 31);
    TS_ASSERT_EQUALS(s1.rfind(s3), simple_string::npos);
    TS_ASSERT_EQUALS(s1.rfind(s3.substr(0, 3)), 40);
    TS_ASSERT_EQUALS(s3.rfind(char(0)), 3);

    // simple_string s1 = "the quick brown fox jumps over the lazy dog";
    // simple_string s2 = "foobar";
    TS_ASSERT_EQUALS(s1.find_first_of(s2), 10);      // 'b'
    TS_ASSERT_EQUALS(s1.find_first_of(s2, 11), 11);  // 'r'
    TS_ASSERT_EQUALS(s1.find_first_of("aeiou"), 2);
    TS_ASSERT_EQUALS(s1.find_first_of("aeiou", 10), 12);
    TS_ASSERT_EQUALS(s1.find_first_of("BIGfoobar", 0, 3), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find_first_of("BIGfoobar", 0), 10);
    TS_ASSERT_EQUALS(s1.find_first_of('e'), 2);
    TS_ASSERT_EQUALS(s1.find_first_of('e', 10), 28);

    TS_ASSERT_EQUALS(s1.find_last_of(s2), 41);      // 'o'
    TS_ASSERT_EQUALS(s1.find_last_of(s2, 11), 11);  // 'r'
    TS_ASSERT_EQUALS(s1.find_last_of("aeiou"), 41);
    TS_ASSERT_EQUALS(s1.find_last_of("aeiou", 10), 6);
    TS_ASSERT_EQUALS(s1.find_last_of("BIGfoobar", simple_string::npos, 3), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find_last_of("BIGfoobar", simple_string::npos), 41);
    TS_ASSERT_EQUALS(s1.find_last_of('e'), 33);
    TS_ASSERT_EQUALS(s1.find_last_of('e', 10), 2);

    // simple_string s1 = "the quick brown fox jumps over the lazy dog";
    // simple_string s2 = "foobar";
    TS_ASSERT_EQUALS(s1.find_first_not_of("the quick"), 10);
    TS_ASSERT_EQUALS(s1.find_first_not_of("the quick", 10), 10);
    TS_ASSERT_EQUALS(s2.find_first_not_of(s1), simple_string::npos);
    TS_ASSERT_EQUALS(s1.find_first_not_of('t'), 1);
    TS_ASSERT_EQUALS(s1.find_first_not_of("the quick", 0, 4), 4);
    TS_ASSERT_EQUALS(s1.find_first_not_of('q', 5), 5);
    
    TS_ASSERT_EQUALS(s1.find_last_not_of("lazy dog"), 33);
    TS_ASSERT_EQUALS(s1[33], 'e');
    TS_ASSERT_EQUALS(s1.find_last_not_of("lazy dog", 37), 33);
    TS_ASSERT_EQUALS(s2.find_last_not_of(s1), simple_string::npos);
    TS_ASSERT_EQUALS(s2.find_last_not_of(s1.substr(10, 2)), 4);
    TS_ASSERT_EQUALS(s1.find_last_not_of('o'), 42);
    TS_ASSERT_EQUALS(s1.find_last_not_of('o', 42), 42);
    TS_ASSERT_EQUALS(s1.find_last_not_of('o', 41), 40);
    TS_ASSERT_EQUALS(s1.find_last_not_of("dog lazy", simple_string::npos, 4), 38);

    s1 = "pentgnddekeder";
    TS_ASSERT_EQUALS(s1.find(''), 4);
    TS_ASSERT_EQUALS(s1.find("ke"), 12);
    TS_ASSERT_EQUALS(s1.find_first_not_of("gpent"), 8);
    TS_ASSERT_EQUALS(s1.find('', 9), 13);
}
